package com.cheesr.layers.domain.order;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Version;

import org.hibernate.annotations.IndexColumn;

import com.cheesr.layers.domain.product.Product;

@Entity
@Table(name = "ORDERS")
public class Order {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @Version
    private Long version;

    @Basic
    private boolean ordered;

    @Basic
    private boolean fulfilled;

    @Basic
    private boolean cancelled;

    @Basic
    private String email;

    @Basic
    private String coupon;

    @OneToOne(cascade = CascadeType.ALL)
    private Address shippingAddress;

    @ManyToMany(targetEntity = Product.class, fetch = FetchType.EAGER)
    @IndexColumn(name = "seq")
    private List<Product> orderProducts = new ArrayList<Product>();

    public Order() {
        shippingAddress = new Address();
    }

    public void addProduct(Product product) {
        isNotAlreadyCheckedOut();
        orderProducts.add(product);
    }

    public void removeProduct(Product product) {
        orderProducts.remove(product);
    }

    public long getId() {
        return id;
    }

    public long getVersion() {
        return version;
    }

    public boolean isOrdered() {
        return ordered;
    }

    public boolean isFulfilled() {
        return fulfilled;
    }

    public boolean isCancelled() {
        return cancelled;
    }

    public List<Product> getOrderProducts() {
        return orderProducts;
    }

    public Address getShippingAddress() {
        return shippingAddress;
    }

    public void checkout() {
        isNotAlreadyCheckedOut();
        hasCompleteAddress();
        hasProducts();
        ordered = true;
    }

    public void fulfill() {
        checkIfOrderable();
        fulfilled = true;
    }

    public void cancel() {
        checkIfOrderable();
        cancelled = true;
    }

    private void checkIfOrderable() {
        if (!isOrdered()) {
            throw new RuntimeException("Not yet ordered");
        }
        if (isFulfilled()) {
            throw new RuntimeException("Already fulfilled");
        }
        if (isCancelled()) {
            throw new RuntimeException("Already cancelled");
        }
    }

    private void hasProducts() {
        if (orderProducts.isEmpty()) {
            throw new RuntimeException("No Ordered products");
        }
    }

    private void hasCompleteAddress() {
        if (!shippingAddress.isValid()) {
            throw new RuntimeException("Shipping adress is incomplete.");
        }
    }

    private void isNotAlreadyCheckedOut() {
        if (isOrdered()) {
            throw new RuntimeException("This order is already checkedOut");
        }
    }

    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof Order) && (this.id != null)) {
            return ((Order) obj).id.equals(this.id);
        } else {
            return super.equals(obj);
        }
    }

    public String getCoupon() {
        return coupon;
    }

    public void setCoupon(String coupon) {
        this.coupon = coupon;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    @Override
    public int hashCode() {
        return (this.id == null ? super.hashCode() : id.intValue());
    }

    public double getTotal() {
        double total = 0;
        for (Product cheese : orderProducts) {
            total += cheese.getPrice();
        }
        return total;
    }
}
